using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace View_by_Distance.Rename.Models.Binder;

public class RenameConfiguration
{

    public string? DefaultMaker { get; set; }
    public bool? ForceNewId { get; set; }
    public string[]? IgnoreExtensions { get; set; }
    public bool? OnlySaveIdentifiersToDisk { get; set; }
    public string? RelativePropertyCollectionFile { get; set; }
    public string[]? SidecarExtensions { get; set; }
    public bool? SkipIdFiles { get; set; }
    public string[]? ValidImageFormatExtensions { get; set; }
    public string[]? ValidVideoFormatExtensions { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderConfigurationSourceGenerationContext.Default.RenameConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, RenameConfiguration? configuration)
    {
        if (configuration?.DefaultMaker is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(RenameConfiguration configuration)
    {
        if (configuration.IgnoreExtensions is null || configuration.IgnoreExtensions.Length == 0) throw new NullReferenceException(nameof(configuration.IgnoreExtensions));
        if (configuration.ValidImageFormatExtensions is null || configuration.ValidImageFormatExtensions.Length == 0) throw new NullReferenceException(nameof(configuration.ValidImageFormatExtensions));
        if (configuration.ValidVideoFormatExtensions is null || configuration.ValidVideoFormatExtensions.Length == 0) throw new NullReferenceException(nameof(configuration.ValidVideoFormatExtensions));
    }

    private static Models.RenameConfiguration Get(RenameConfiguration? configuration, Shared.Models.MetadataConfiguration metadataConfiguration)
    {
        Models.RenameConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.DefaultMaker is null) throw new NullReferenceException(nameof(configuration.DefaultMaker));
        if (configuration.ForceNewId is null) throw new NullReferenceException(nameof(configuration.ForceNewId));
        if (configuration.IgnoreExtensions is null) throw new NullReferenceException(nameof(configuration.IgnoreExtensions));
        if (configuration.OnlySaveIdentifiersToDisk is null) throw new NullReferenceException(nameof(configuration.OnlySaveIdentifiersToDisk));
        if (configuration.RelativePropertyCollectionFile is null) throw new NullReferenceException(nameof(configuration.RelativePropertyCollectionFile));
        if (configuration.SidecarExtensions is null) throw new NullReferenceException(nameof(configuration.SidecarExtensions));
        if (configuration.SkipIdFiles is null) throw new NullReferenceException(nameof(configuration.SkipIdFiles));
        if (configuration.ValidImageFormatExtensions is null) throw new NullReferenceException(nameof(configuration.ValidImageFormatExtensions));
        if (configuration.ValidVideoFormatExtensions is null) throw new NullReferenceException(nameof(configuration.ValidVideoFormatExtensions));
        Verify(configuration);
        result = new(metadataConfiguration,
                     configuration.DefaultMaker,
                     configuration.ForceNewId.Value,
                     configuration.IgnoreExtensions,
                     configuration.OnlySaveIdentifiersToDisk.Value,
                     configuration.RelativePropertyCollectionFile,
                     configuration.SidecarExtensions,
                     configuration.SkipIdFiles.Value,
                     configuration.ValidImageFormatExtensions,
                     configuration.ValidVideoFormatExtensions);
        return result;
    }

    public static Models.RenameConfiguration Get(IConfigurationRoot configurationRoot, Shared.Models.MetadataConfiguration metadataConfiguration)
    {
        Models.RenameConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(RenameConfiguration));
#pragma warning disable IL3050, IL2026
        RenameConfiguration? configuration = configurationSection.Get<RenameConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration, metadataConfiguration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(RenameConfiguration))]
internal partial class BinderConfigurationSourceGenerationContext : JsonSerializerContext
{
}